"""
test_project URL Configuration
"""
from django.conf.urls import url
from django.contrib import admin
from django.http import HttpResponse

urlpatterns = [
    url(r"^homepage/", lambda r: HttpResponse("HELLO"), name="homepage"),
    url(r"^my_view/", "test_project.views.my_view"),
    url(r"^admin/", admin.site.urls, name="admin"),
]





from django.template import Library, defaulttags

from django_compat_patcher.deprecation import *

register = Library()


@register.tag
def cycle(parser, token):
    """
    This is the future version of `cycle` with auto-escaping.
    The deprecation is now complete and this version is no different
    from the non-future version so this is deprecated.

    By default all strings are escaped.

    If you want to disable auto-escaping of variables you can use::

        {% autoescape off %}
            {% cycle var1 var2 var3 as somecycle %}
        {% autoescape %}

    Or if only some variables should be escaped, you can use::

        {% cycle var1 var2|safe var3|safe  as somecycle %}
    """
    warnings.warn(
        "Loading the `cycle` tag from the `future` library is deprecated and "
        "will be removed in Django 1.10. Use the default `cycle` tag instead.",
        RemovedInDjango110Warning)
    return defaulttags.cycle(parser, token)


@register.tag
def firstof(parser, token):
    """
    This is the future version of `firstof` with auto-escaping.
    The deprecation is now complete and this version is no different
    from the non-future version so this is deprecated.

    This is equivalent to::

        {% if var1 %}
            {{ var1 }}
        {% elif var2 %}
            {{ var2 }}
        {% elif var3 %}
            {{ var3 }}
        {% endif %}

    If you want to disable auto-escaping of variables you can use::

        {% autoescape off %}
            {% firstof var1 var2 var3 "<strong>fallback value</strong>" %}
        {% autoescape %}

    Or if only some variables should be escaped, you can use::

        {% firstof var1 var2|safe var3 "<strong>fallback value</strong>"|safe %}
    """
    warnings.warn(
        "Loading the `firstof` tag from the `future` library is deprecated and "
        "will be removed in Django 1.10. Use the default `firstof` tag instead.",
        RemovedInDjango110Warning)
    return defaulttags.firstof(parser, token)



from __future__ import absolute_import, print_function, unicode_literals

from compat_patcher_core.registry import PatchingRegistry


class DjangoPatchingRegistry(PatchingRegistry):
    pass  # Add hre custom behaviours for the Django registry

    def register_compatibility_fixer(self, *args, **kwargs):
        """Helper override to easily handle the "fixer_delayed" tag of Django-specific fixers."""
        fixer_delayed = kwargs.pop("fixer_delayed", None)
        if fixer_delayed:
            kwargs.setdefault("fixer_tags", []).append("fixer_delayed")
        return super(DjangoPatchingRegistry, self).register_compatibility_fixer(
            *args, **kwargs
        )


def get_current_django_version():
    import django

    return django.get_version()


def populate_django_fixers_registry(registry):
    import django_compat_patcher.fixers


# Must be instantiated HERE so that fixer submodules can access it at import time
django_patching_registry = DjangoPatchingRegistry(
    family_prefix="django",
    populate_callable=populate_django_fixers_registry,
    current_software_version=get_current_django_version,
)

register_django_compatibility_fixer = (
    django_patching_registry.register_compatibility_fixer
)  # Shortcut



from __future__ import absolute_import, print_function, unicode_literals

import _test_utilities


def test_fix_behaviour_widget_build_attrs():
    from django.forms.widgets import Select

    select = Select({"someattr": "somevalue"})

    attrs = select.build_attrs({"other": "yes"}, multiple="multiple")  # old behavior
    assert attrs == {"someattr": "somevalue", "multiple": "multiple", "other": "yes"}

    try:
        attrs = select.build_attrs({"this": "no"}, {"other": "yes"})  # new behavior
        assert attrs == {"this": "no", "other": "yes"}
    except TypeError:  # old (unpatched) signature, with only 2 positional arguments including "self"
        attrs = select.build_attrs(extra_attrs={"this": "no"})
        assert attrs == {"someattr": "somevalue", "this": "no"}
    else:
        attrs = select.build_attrs({"this": "no"})  # new behavior
        assert attrs == {"this": "no"}  # no self.attrs added



from __future__ import absolute_import, print_function, unicode_literals

from functools import partial

from django.core.exceptions import ImproperlyConfigured
from django.utils import six

from ..deprecation import *
from ..registry import register_django_compatibility_fixer

# for backward-compatibility fixers
django1_10_bc_fixer = partial(
    register_django_compatibility_fixer,
    fixer_reference_version="1.10",
    fixer_applied_from_version="1.10",
)


def _get_url_utils():
    """
    Get URL utilities through versions, despite them being moved and refactored (with new "path()" syntax).
    """
    try:
        from django.urls import (
            get_callable,
            RegexURLPattern,
            URLPattern,
            RegexURLResolver,
            URLResolver,
            NoReverseMatch,
        )
    except ImportError:
        # previously, there was no "RoutePattern vs RegexPattern"
        from django.core.urlresolvers import (
            get_callable,
            RegexURLPattern,
            RegexURLPattern as URLPattern,
            RegexURLResolver,
            RegexURLResolver as URLResolver,
            NoReverseMatch,
        )  # old location
    return (
        get_callable,
        RegexURLPattern,
        URLPattern,
        RegexURLResolver,
        URLResolver,
        NoReverseMatch,
    )


@register_django_compatibility_fixer(
    fixer_reference_version="1.10", fixer_applied_upto_version="1.10"
)
def fix_incoming_urls_submodule(utils):
    """
    Put a forward compatibility import path for django.urls, which replaces django.core.urlresolvers
    """
    from django.core import urlresolvers

    utils.inject_module("django.urls", urlresolvers)


@django1_10_bc_fixer()
def fix_deletion_templatetags_future(utils):
    """
    Preserve the "future" templatetags library, with its improved `firstof` and `cycle` tags.
    """
    import django.templatetags
    from ..django_legacy.django1_10.templatetags import future

    utils.inject_module("django.templatetags.future", future)
    utils.inject_attribute(django.templatetags, "future", future)

    from django.template.backends import django as django_templates

    _old_get_installed_libraries = django_templates.get_installed_libraries

    def get_installed_libraries():
        libraries = (
            _old_get_installed_libraries()
        )  # tries real __import__() calls on submodules
        libraries["future"] = "django.templatetags.future"
        # print(">>>>> FINAL libraries", libraries)
        return libraries

    utils.inject_callable(
        django_templates, "get_installed_libraries", get_installed_libraries
    )


@django1_10_bc_fixer()
def fix_deletion_template_defaulttags_ssi(utils):
    """
    Preserve the "ssi" default template tag.
    """
    import django.template.defaulttags
    from ..django_legacy.django1_10.template import defaulttags

    utils.inject_callable(
        django.template.defaulttags,
        "include_is_allowed",
        defaulttags.include_is_allowed,
    )
    utils.inject_class(django.template.defaulttags, "SsiNode", defaulttags.SsiNode)
    utils.inject_callable(django.template.defaulttags, "ssi", defaulttags.ssi)
    django.template.defaulttags.register.tag(defaulttags.ssi)

    from django.template.engine import Engine

    _old_init = Engine.__init__

    def __init__(self, dirs=None, app_dirs=False, allowed_include_roots=None, **kwargs):
        if allowed_include_roots is None:
            allowed_include_roots = []
        if isinstance(allowed_include_roots, six.string_types):
            raise ImproperlyConfigured(
                "allowed_include_roots must be a tuple, not a string."
            )
        self.allowed_include_roots = allowed_include_roots
        _old_init(self, dirs=dirs, app_dirs=app_dirs, **kwargs)

    utils.inject_callable(Engine, "__init__", __init__)


@django1_10_bc_fixer()
def fix_behaviour_urls_resolvers_RegexURLPattern(utils):
    """
    Restore support for dotted-string view parameter in RegexURLPattern, instead passing a view object.
    """

    get_callable, RegexURLPattern, URLPattern, RegexURLResolver, URLResolver, NoReverseMatch = (
        _get_url_utils()
    )
    del RegexURLPattern  # we work on the common URLPattern class

    @property
    def callback(self):
        callback = self.__dict__["callback"]  # bypass descriptor
        if isinstance(callback, six.string_types):
            callback_obj = get_callable(callback)
        else:
            callback_obj = callback
        return callback_obj

    @callback.setter
    def callback(self, value):
        self.__dict__["callback"] = value  # bypass descriptor

    # we inject a DATA-DESCRIPTOR, so it'll be accessed in prority
    # over "self.callback" instance attribute
    utils.inject_attribute(URLPattern, "callback", callback)

    def add_prefix(self, prefix):
        """
        Adds the prefix string to a string-based callback.
        """
        callback = self.__dict__["callback"]
        if not prefix or not isinstance(callback, six.string_types):
            return
        self.callback = prefix + "." + callback

    utils.inject_callable(URLPattern, "add_prefix", add_prefix)

    original_lookup_str = URLPattern.lookup_str

    @property  # not cached...
    def lookup_str(self):
        callback = self.__dict__["callback"]
        if isinstance(callback, six.string_types):
            # no need for warning, already emitted above
            return callback  # already a dotted path to view
        return original_lookup_str.__get__(self, self.__class__)

    utils.inject_attribute(URLPattern, "lookup_str", lookup_str)


@django1_10_bc_fixer()
def fix_behaviour_core_urlresolvers_reverse_with_prefix(utils):
    """
    Preserve the ability to call urlresolver on dotted string view,
    instead of explicit view name.
    """

    get_callable, RegexURLPattern, URLPattern, RegexURLResolver, URLResolver, NoReverseMatch = (
        _get_url_utils()
    )
    del RegexURLResolver  # we patch the common URLResolver class

    original_reverse_with_prefix = URLResolver._reverse_with_prefix

    def _reverse_with_prefix(self, lookup_view, _prefix, *args, **kwargs):
        original_lookup = lookup_view
        try:
            if self._is_callback(lookup_view):
                utils.emit_warning(
                    "Reversing by dotted path is deprecated (%s)." % original_lookup,
                    RemovedInDjango110Warning,
                    stacklevel=3,
                )
                lookup_view = get_callable(lookup_view)
        except (ImportError, AttributeError) as e:
            raise NoReverseMatch("Error importing '%s': %s." % (lookup_view, e))
        return original_reverse_with_prefix(self, lookup_view, _prefix, *args, **kwargs)

    utils.inject_callable(URLResolver, "_reverse_with_prefix", _reverse_with_prefix)


@django1_10_bc_fixer()
def fix_behaviour_conf_urls_url(utils):
    """
    Support passing views to url() as dotted strings instead of view objects.
    """
    get_callable, RegexURLPattern, URLPattern, RegexURLResolver, URLResolver, NoReverseMatch = (
        _get_url_utils()
    )
    del URLPattern  # we stick to the old "regex-only" URL system

    from django.conf import urls

    def url(regex, view, kwargs=None, name=None, prefix=""):
        if isinstance(view, (list, tuple)):
            # For include(...) processing.
            urlconf_module, app_name, namespace = view
            return RegexURLResolver(
                regex, urlconf_module, kwargs, app_name=app_name, namespace=namespace
            )
        else:
            if isinstance(view, six.string_types):
                utils.emit_warning(
                    "Support for string view arguments to url() is deprecated and "
                    "will be removed in Django 1.10 (got %s). Pass the callable "
                    "instead." % view,
                    RemovedInDjango110Warning,
                    stacklevel=2,
                )
                if not view:
                    raise ImproperlyConfigured(
                        "Empty URL pattern view name not permitted (for pattern %r)"
                        % regex
                    )
                if prefix:
                    view = prefix + "." + view
            return RegexURLPattern(regex, view, kwargs, name)

    assert callable(urls.url)
    utils.inject_callable(urls, "url", url)


@django1_10_bc_fixer()
def fix_deletion_conf_urls_patterns(utils):
    """
    Preserve the patterns() builder for django urls.
    """
    from django.conf import urls

    get_callable, RegexURLPattern, URLPattern, RegexURLResolver, URLResolver, NoReverseMatch = (
        _get_url_utils()
    )
    del URLPattern  # we stick to the old "regex-only" URL system

    def patterns(prefix, *args):
        utils.emit_warning(
            "django.conf.urls.patterns() is deprecated and will be removed in "
            "Django 1.10. Update your urlpatterns to be a list of "
            "django.conf.urls.url() instances instead.",
            RemovedInDjango110Warning,
            stacklevel=2,
        )
        pattern_list = []
        for t in args:
            if isinstance(t, (list, tuple)):
                t = urls.url(prefix=prefix, *t)
            elif isinstance(t, RegexURLPattern):
                t.add_prefix(prefix)
            pattern_list.append(t)
        return pattern_list

    utils.inject_callable(urls, "patterns", patterns)

    urls.__all__.append(str("patterns"))  # so that star imports work fine


@django1_10_bc_fixer()
def fix_behaviour_template_smartif_OPERATORS_equals(utils):
    """
    Preserve support for a single '=' sign in {% if %} tag.
    """
    from django.template import smartif

    smartif.OPERATORS["="] = smartif.OPERATORS["=="]  # operator alias


''' REQUIRES PYTHON >= 3.3
@django1_10_bc_fixer()
def fix_deletion_core_context_processors(utils):
    """
    Keep django.core.context_processors middlewares as aliases for
    those of django.template.context_processors.
    """
    from .. import import_proxifier
    import_proxifier.install_module_alias_finder()  # idempotent
    import_proxifier.register_module_alias(module_alias="django.core.context_processors middlewares",
                                           real_module="django.template.context_processors")
'''



from __future__ import absolute_import, print_function, unicode_literals

from functools import partial

from ..deprecation import *
from ..registry import register_django_compatibility_fixer

# for backward-compatibility fixers
django1_21_bc_fixer = partial(
    register_django_compatibility_fixer,
    fixer_reference_version="2.1",
    fixer_applied_from_version="2.1",
)


@django1_21_bc_fixer()
def fix_deletion_utils_translation_string_concat(utils):
    """
    Preserve django.utils.translation.string_concat(), superseded by django.utils.text.format_lazy().
    """
    import django.utils.translation
    from django.utils.functional import lazy

    def _string_concat(*strings):
        """
        Lazy variant of string concatenation, needed for translations that are
        constructed from multiple parts.
        """
        utils.emit_warning(
            "django.utils.translate.string_concat() is deprecated in "
            "favor of django.utils.text.format_lazy().",
            RemovedInDjango21Warning,
            stacklevel=2,
        )
        return "".join(str(s) for s in strings)

    string_concat = lazy(_string_concat, str)
    utils.inject_callable(django.utils.translation, "string_concat", string_concat)


@django1_21_bc_fixer()
def fix_behaviour_widget_render_forced_renderer(utils):
    """
    Restore the behaviour where the "renderer" parameter of Widget.render() may not be supported by subclasses.
    """
    from django.forms.boundfield import BoundField

    original_as_widget = BoundField.as_widget

    def as_widget(self, widget=None, attrs=None, only_initial=False):

        widget = widget or self.field.widget

        from django.utils.inspect import func_supports_parameter, func_accepts_kwargs

        if not (
            func_supports_parameter(widget.render, "renderer")
            or func_accepts_kwargs(widget.render)
        ):
            original_widget_render = widget.render

            utils.emit_warning(
                "Add the `renderer` argument to the render() method of %s. "
                "It will be mandatory in Django 2.1." % widget.__class__,
                RemovedInDjango21Warning,
                stacklevel=2,
            )

            def instance_render(name, value, attrs=None, renderer=None):
                del renderer  # restore non-mandatory support for this parameter
                return original_widget_render(name=name, value=value, attrs=attrs)

            utils.inject_callable(
                widget, "render", instance_render
            )  # beware, function stored in INSTANCE

        return original_as_widget(
            self, widget=widget, attrs=attrs, only_initial=only_initial
        )

    utils.inject_callable(BoundField, "as_widget", as_widget)



from __future__ import absolute_import, print_function, unicode_literals

import _test_utilities


def test_fix_deletion_http_request_HttpRequest_raw_post_data():
    from django.core.handlers.wsgi import WSGIRequest
    from django.test.client import FakePayload

    payload = FakePayload("Hello There!")
    request = WSGIRequest(
        {
            "REQUEST_METHOD": "POST",
            "CONTENT_LENGTH": len(payload),
            "wsgi.input": payload,
        }
    )

    assert request.body == request.raw_post_data
    assert request.body == b"Hello There!"



from __future__ import absolute_import, print_function, unicode_literals

import _test_utilities


def test_fix_deletion_utils_translation_string_concat():
    from django.utils.translation import string_concat

    assert string_concat("a", "b", "c") == "abc"


def test_fix_behaviour_widget_render_forced_renderer():

    from django import forms
    from django.forms.forms import BoundField  # not propagated in django1.8

    class MyForm(forms.Form):
        my_field = forms.CharField(label="My field", max_length=100)

    class OldWidget(forms.Widget):
        def render(self, name, value, attrs=None):  # no "renderer" parameter expected
            return "All is ok 1"

    class NewWidget(forms.Widget):
        def render(self, name, value, attrs=None, renderer=None):
            return "All is ok 2"

    form = MyForm()

    my_field = form["my_field"]
    assert isinstance(my_field, BoundField)

    res = my_field.as_widget(OldWidget())
    assert res == "All is ok 1"

    res = my_field.as_widget(NewWidget())
    assert res == "All is ok 2"

    res = my_field.as_widget()  # implicit CharField widget
    assert "<input " in res



def pytest_collection_modifyitems(config, items):
    from django_compat_patcher.registry import django_patching_registry
    from compat_patcher_core.scaffolding import (
        ensure_all_fixers_have_a_test_under_pytest,
    )

    ensure_all_fixers_have_a_test_under_pytest(
        config=config, items=items, patching_registry=django_patching_registry
    )



from __future__ import absolute_import, print_function, unicode_literals

import os
import sys

sys.path.append(os.path.dirname(os.path.abspath(__file__)))

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "test_project.settings_no_db")

import django
import django_compat_patcher

# This MUST happen BEFORE django is setup(), else import proxies can't be installed
django_compat_patcher.patch()

django.setup()  # idempotent

DJANGO_VERSION_TUPLE = django.VERSION



from __future__ import absolute_import, print_function, unicode_literals

class DjangoSettingsProvider(object):

    _settings = None

    def __init__(self, settings):

        if settings:
            self._settings = settings
        else:
            from django.conf import settings as django_settings

            self._settings = django_settings.__dict__

        from . import default_settings as default_settings

        self._fallback_settings = default_settings.__dict__

    def __getitem__(self, item):
        # Might raise KeyError if setting not existing at all

        item = "DCP_" + item.upper()  # Turn to Django naming convention

        try:
            value = self._settings[item]
        except KeyError as e:
            # print("EXCEPTION IN get_patcher_setting", name, e)
            value = self._fallback_settings[item]
        return value



"""
Django settings for test_project project.

Generated by 'django-admin startproject' using Django 1.9.6.
"""

import django

django_version = django.VERSION

from .minimal_settings import *


SITE_ID = 1

# Application definition

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.sites",
    "test_project",
]

if not os.environ.get("IGNORE_CONTRIB_COMMENTS"):
    # old contrib packages, restored by proxifier in their original django.contrib.* namespace
    INSTALLED_APPS.append(
        "django_comments" if django_version >= (1, 8) else "django.contrib.comments"
    )


MIDDLEWARE_CLASSES = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.auth.middleware.SessionAuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

ROOT_URLCONF = "test_project.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "allowed_include_roots": [BASE_DIR],
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    }
]

WSGI_APPLICATION = "test_project.wsgi.application"


# Database
# https://docs.djangoproject.com/en/1.9/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
    }
}


# Internationalization
# https://docs.djangoproject.com/en/1.9/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.9/howto/static-files/

STATIC_URL = "/static/"



from __future__ import absolute_import, print_function, unicode_literals

from compat_patcher_core.utilities import WarningsProxy

# Proxy meant to be imported by fixer submodules instead of stdlib "warnings" package
warnings = WarningsProxy()


class RemovedInDjango22Warning(DeprecationWarning):
    pass


class RemovedInDjango21Warning(DeprecationWarning):
    pass


class RemovedInDjango20Warning(DeprecationWarning):
    pass


class RemovedInDjango111Warning(DeprecationWarning):
    pass


class RemovedInDjango110Warning(DeprecationWarning):
    pass


class RemovedInDjango19Warning(DeprecationWarning):
    pass


class RemovedInDjango18Warning(DeprecationWarning):
    pass


class RemovedInDjango17Warning(DeprecationWarning):
    pass


class RemovedInDjango16Warning(DeprecationWarning):
    pass


# Beware, think about updating this one!
RemovedInNextVersionWarning = RemovedInDjango22Warning



from __future__ import absolute_import, print_function, unicode_literals

import os
import pytest

import _test_utilities


@pytest.mark.skipif(
    _test_utilities.DJANGO_VERSION_TUPLE < (1, 10),
    reason="requires django.urls subpackage",
)
def test_fix_deletion_urls_RegexURLPattern_RegexURLResolver():

    from django.urls import RegexURLPattern
    from django.urls.resolvers import RegexURLPattern as RegexURLPattern2

    assert RegexURLPattern is RegexURLPattern2

    url = RegexURLPattern("^mypage\d", lambda x: x, name="mygoodurl")
    if hasattr(url, "check"):
        assert not url.check()
    assert url.resolve("mypage3")
    assert not url.resolve("mypage")

    from django.urls import RegexURLResolver
    from django.urls.resolvers import RegexURLResolver as RegexURLResolver2

    assert RegexURLResolver is RegexURLResolver2

    from django.conf import settings

    urlconf = settings.ROOT_URLCONF
    resolver = RegexURLResolver(r"^/", urlconf)
    if hasattr(resolver, "check"):
        assert not resolver.check()
    assert resolver.resolve("/homepage/")
    from django.urls import Resolver404

    with pytest.raises(Resolver404):
        resolver.resolve("/homepageXXX/")


@pytest.mark.skipif(
    _test_utilities.DJANGO_VERSION_TUPLE < (1, 10),
    reason="requires django.urls subpackage",
)
def test_fix_deletion_core_urlresolvers():

    from django.urls import get_resolver
    from django.core.urlresolvers import get_resolver as get_resolver2

    assert get_resolver is get_resolver2


def test_fix_deletion_template_library_assignment_tag():
    from django import template

    register = template.Library()

    @register.assignment_tag
    def mytag():
        return "mycontent"

    assert mytag() == "mycontent"


def test_fix_deletion_utils_functional_allow_lazy():
    import six
    from django.utils.encoding import force_text
    from django.utils.functional import allow_lazy, lazy

    def myfunc(arg):
        return arg

    myfunc = allow_lazy(myfunc, six.text_type)

    proxy = myfunc(lazy(force_text, six.text_type)("mystr"))
    assert type(proxy) != str

    value = proxy.__str__()
    assert value == "mystr"


def test_fix_deletion_template_context_Context_has_key():
    from django.template.context import Context

    ctx = Context({"a": 65})
    assert ctx.has_key("a")
    assert not ctx.has_key("b")


@pytest.mark.skipif(
    _test_utilities.DJANGO_VERSION_TUPLE < (1, 9), reason="Requires json_catalog() view"
)
def test_fix_deletion_views_i18n_javascript_and_json_catalog():
    from django.views.i18n import (
        javascript_catalog,
        json_catalog,
        render_javascript_catalog,
        null_javascript_catalog,
    )
    from django.http import HttpResponse
    from django.test.client import RequestFactory

    request = RequestFactory().get("/homepage/")

    response = javascript_catalog(request)
    assert isinstance(response, HttpResponse)

    response = json_catalog(request)
    assert isinstance(response, HttpResponse)

    response = null_javascript_catalog(request)
    assert isinstance(response, HttpResponse)

    response = render_javascript_catalog({"chateau": "castle"})
    assert isinstance(response, HttpResponse)


@pytest.mark.skipif(
    _test_utilities.DJANGO_VERSION_TUPLE < (2, 0),
    reason="Requires field.remote_field attribute",
)
def test_fix_behaviour_db_models_fields_related_ForeignKey_OneToOneField():

    from django.contrib.auth import get_user_model
    from django.db.models import ForeignKey, OneToOneField, CASCADE, PROTECT

    User = get_user_model()

    fk = ForeignKey(User)
    assert fk.remote_field.on_delete == CASCADE
    del fk

    fk = ForeignKey(User, on_delete=PROTECT)
    assert fk.remote_field.on_delete == PROTECT
    del fk

    fk = OneToOneField(User)
    assert fk.remote_field.on_delete == CASCADE
    del fk

    fk = OneToOneField(User, on_delete=PROTECT)
    assert fk.remote_field.on_delete == PROTECT
    del fk


def test_fix_behaviour_conf_urls_include_3tuples():
    from django.conf.urls import include

    try:
        from django.urls import include as include2

        assert include is include2
    except ImportError:  # normal for old Django versions
        if _test_utilities.DJANGO_VERSION_TUPLE >= (2, 0):
            raise

    from django.contrib import admin

    assert len(admin.site.urls) == 3
    include(admin.site.urls)  # is OK as a 3-tuple

    from django.core.exceptions import ImproperlyConfigured

    with pytest.raises(ImproperlyConfigured):
        include(admin.site.urls, namespace="mynamespace")



from __future__ import absolute_import, print_function, unicode_literals

import os

import pytest

import _test_utilities


def test_fix_outsourcing_contrib_comments():

    if os.environ.get(
        "IGNORE_CONTRIB_COMMENTS"
    ):  # case where external dependency "django_comments" isn't loaded

        with pytest.raises(ImportError) as exc:
            from django.contrib import comments
        assert (
            "No module named 'django_comments' (when loading alias name 'django.contrib.comments')"
            in str(exc)
        )

    else:

        from django.contrib import comments
        import django.contrib.comments

        import django.contrib.comments.urls

        assert isinstance(django.contrib.comments.urls.urlpatterns, list)

        from django.contrib.comments.views import comments as comments_views

        assert callable(comments_views.post_comment)



from __future__ import absolute_import, print_function, unicode_literals

# Filters applied first, the special value "*" means "all"
DCP_INCLUDE_FIXER_IDS = "*"
DCP_INCLUDE_FIXER_FAMILIES = []

# Filters applied last
DCP_EXCLUDE_FIXER_IDS = []
DCP_EXCLUDE_FIXER_FAMILIES = []

DCP_PATCH_INJECTED_OBJECTS = "__dcp_injected__"

DCP_LOGGING_LEVEL = "INFO"
DCP_ENABLE_WARNINGS = True



from __future__ import absolute_import, print_function, unicode_literals

from functools import partial

from ..deprecation import *
from ..registry import register_django_compatibility_fixer

# for backward-compatibility fixers
django1_9_bc_fixer = partial(
    register_django_compatibility_fixer,
    fixer_reference_version="1.9",
    fixer_applied_from_version="1.9",
)


@django1_9_bc_fixer()
def fix_deletion_utils_datastructures_MergeDict(utils):
    """
    Preserve the MergeDict util datastructure
    """
    from django.utils import datastructures as dj_datastructures
    from ..django_legacy.django1_9.datastructures import MergeDict as MergeDictCompat

    utils.inject_class(dj_datastructures, "MergeDict", MergeDictCompat)


@django1_9_bc_fixer()
def fix_deletion_utils_datastructures_SortedDict(utils):
    """
    Preserve the SortedDict util datastructure
    """
    from django.utils import datastructures as dj_datastructures
    from ..django_legacy.django1_9.datastructures import SortedDict as SortedDictCompat

    utils.inject_class(dj_datastructures, "SortedDict", SortedDictCompat)


@django1_9_bc_fixer()
def fix_deletion_utils_dictconfig(utils):
    """
    Preserve the dictconfig util file
    """
    import django.utils
    from django_compat_patcher.django_legacy.django1_9.utils import dictconfig

    utils.inject_module("django.utils.dictconfig", dictconfig)
    utils.inject_attribute(django.utils, "dictconfig", dictconfig)


@django1_9_bc_fixer()
def fix_deletion_utils_functional_memoize(utils):
    """
    Preserve utils.functional.memoize() utility
    """
    import django.utils.functional
    from django_compat_patcher.django_legacy.django1_9.utils.functional import memoize

    utils.inject_callable(django.utils.functional, "memoize", memoize)


@django1_9_bc_fixer()
def fix_deletion_utils_importlib(utils):
    """
    Preserve the importlib util file
    """
    import django.utils
    from django_compat_patcher.django_legacy.django1_9.utils import importlib

    utils.inject_module("django.utils.importlib", importlib)
    utils.inject_attribute(django.utils, "importlib", importlib)


@django1_9_bc_fixer()
def fix_deletion_utils_tzinfo(utils):
    """
    Preserve the tzinfo util file
    """
    import django.utils
    from django_compat_patcher.django_legacy.django1_9.utils import tzinfo

    utils.inject_module("django.utils.tzinfo", tzinfo)
    utils.inject_attribute(django.utils, "tzinfo", tzinfo)


@django1_9_bc_fixer()
def fix_deletion_utils_unittest(utils):
    """
    Preserve the unittest util file
    """
    import django.utils
    from django_compat_patcher.django_legacy.django1_9.utils import unittest

    utils.inject_module("django.utils.unittest", unittest)
    utils.inject_attribute(django.utils, "unittest", unittest)


@django1_9_bc_fixer()
def fix_deletion_core_handlers_wsgi_WSGIRequest_REQUEST(utils):
    "Preserve the `request.REQUEST` attribute, merging parameters from GET "
    "and POST (the latter has precedence)."

    from django.core.handlers.wsgi import WSGIRequest
    from django.utils.datastructures import MergeDict  # Depends on a previous fixer

    def _get_request(self):
        utils.emit_warning(
            "`request.REQUEST` is deprecated, use `request.GET` or "
            "`request.POST` instead.",
            RemovedInDjango19Warning,
            2,
        )
        if not hasattr(self, "_request"):
            self._request = MergeDict(self.POST, self.GET)
        return self._request

    utils.inject_callable(WSGIRequest, "_get_request", _get_request)
    utils.inject_attribute(WSGIRequest, "REQUEST", property(_get_request))


@django1_9_bc_fixer()
def fix_deletion_contrib_admin_ModelAdmin_get_formsets(utils):
    """
    Preserve the get_formsets method of ModelAdmin
    """
    from django.contrib.admin import ModelAdmin

    def _get_formsets(self, request, obj):
        """
        Helper function that exists to allow the deprecation warning to be
        executed while this function continues to return a generator.
        """
        for inline in self.get_inline_instances(request, obj):
            yield inline.get_formset(request, obj)

    def get_formsets(self, request, obj=None):
        utils.emit_warning(
            "ModelAdmin.get_formsets() is deprecated Use ModelAdmin.get_formsets_with_inlines() instead.",
            RemovedInDjango19Warning,
            stacklevel=2,
        )
        return self._get_formsets(request, obj)

    utils.inject_callable(ModelAdmin, "_get_formsets", _get_formsets)
    utils.inject_callable(ModelAdmin, "get_formsets", get_formsets)


@django1_9_bc_fixer()
def fix_deletion_templatetags_future_url(utils):
    "Preserve the `url` tag in the `future` templatetags library."
    from django.template import defaulttags
    from django.templatetags import future

    new_tag = utils.inject_callable_alias(
        future, "url", source_object=defaulttags, source_attrname="url"
    )
    future.register.tag(new_tag)


@django1_9_bc_fixer()
def fix_deletion_templatetags_future_ssi(utils):
    "Preserve the `ssi` tag in the `future` templatetags library."
    from django.template import defaulttags
    from django.templatetags import future

    new_tag = utils.inject_callable_alias(
        future, "ssi", source_object=defaulttags, source_attrname="ssi"
    )
    future.register.tag(new_tag)


@django1_9_bc_fixer()
def fix_deletion_forms_fields_IPAddressField(utils):
    """Preserve the IPAddressField form field, now superseded by GenericIPAddressField"""
    import django.forms.fields
    from django.forms.fields import CharField
    from django.core import validators

    class IPAddressField(CharField):
        default_validators = [validators.validate_ipv4_address]

        def __init__(self, *args, **kwargs):
            utils.emit_warning(
                "IPAddressField has been deprecated. Use GenericIPAddressField instead.",
                RemovedInDjango19Warning,
            )
            super(IPAddressField, self).__init__(*args, **kwargs)

        def to_python(self, value):
            if value in self.empty_values:
                return ""
            return value.strip()

    utils.inject_class(django.forms.fields, "IPAddressField", IPAddressField)

    from django.db.models.fields import IPAddressField as OriginalIPAddressField

    def formfield(self, **kwargs):
        defaults = {"form_class": django.forms.fields.IPAddressField}
        defaults.update(kwargs)
        return super(OriginalIPAddressField, self).formfield(**defaults)

    utils.inject_callable(OriginalIPAddressField, "formfield", formfield)


@django1_9_bc_fixer()
def fix_deletion_core_management_base_AppCommand_handle_app(utils):
    """Preserve the fallback to AppCommand.handle_app() method in django management commands."""

    from django.core.management.base import CommandError, AppCommand

    def handle_app_config(self, app_config, **options):
        """
        Perform the command's actions for app_config, an AppConfig instance
        corresponding to an application label given on the command line.
        """
        try:
            # During the deprecation path, keep delegating to handle_app if
            # handle_app_config isn't implemented in a subclass.
            handle_app = self.handle_app
        except AttributeError:
            # Keep only this exception when the deprecation completes.
            raise NotImplementedError(
                "Subclasses of AppCommand must provide " "a handle_app_config() method."
            )
        else:
            utils.emit_warning(
                "AppCommand.handle_app() is superseded by "
                "AppCommand.handle_app_config().",
                RemovedInDjango19Warning,
                stacklevel=2,
            )
            if app_config.models_module is None:
                raise CommandError(
                    "AppCommand cannot handle app '%s' in legacy mode "
                    "because it doesn't have a models module." % app_config.label
                )
            return handle_app(app_config.models_module, **options)

    utils.inject_callable(AppCommand, "handle_app_config", handle_app_config)


@django1_9_bc_fixer(fixer_delayed=True)
def fix_deletion_contrib_sites_models_RequestSite(utils):
    """Preserve contrib.sites.models.RequestSite alias."""
    utils.skip_if_app_not_installed("django.contrib.sites")
    import django.contrib.sites.models
    from django.contrib.sites.requests import RequestSite as RealRequestSite

    class RequestSite(RealRequestSite):
        def __init__(self, *args, **kwargs):
            utils.emit_warning(
                "Please import RequestSite from django.contrib.sites.requests.",
                RemovedInDjango19Warning,
                stacklevel=2,
            )
            super(RequestSite, self).__init__(*args, **kwargs)

    utils.inject_class(django.contrib.sites.models, "RequestSite", RequestSite)


@django1_9_bc_fixer(fixer_delayed=True)
def fix_deletion_contrib_sites_models_get_current_site(utils):
    """Preserve contrib.sites.models.get_current_site alias."""
    utils.skip_if_app_not_installed("django.contrib.sites")
    import django.contrib.sites.models
    from django.contrib.sites.shortcuts import get_current_site as real_get_current_site

    def get_current_site(request):
        utils.emit_warning(
            "Please import get_current_site from django.contrib.sites.shortcuts.",
            RemovedInDjango19Warning,
            stacklevel=2,
        )
        return real_get_current_site(request)

    utils.inject_callable(
        django.contrib.sites.models, "get_current_site", get_current_site
    )



from __future__ import absolute_import, print_function, unicode_literals

import os
import sys

sys.path.append(os.path.dirname(os.path.abspath(__file__)))

os.environ["DJANGO_SETTINGS_MODULE"] = "test_project.minimal_settings"

import django
import django_compat_patcher

django_compat_patcher.patch()  # fixers which can't apply should be skipped, not crash

django.setup()  # idempotent

from django.conf import settings

assert settings.INSTALLED_APPS == []  # we ensure that we haven't mixed django confs



from __future__ import absolute_import, print_function, unicode_literals

from compat_patcher_core import generic_patch_software, make_safe_patcher


@make_safe_patcher
def patch(settings=None):
    """Load every dependency, and apply registered fixers according to provided settings (or Django settings as a fallback)."""

    from .registry import django_patching_registry
    from .deprecation import warnings as warnings_proxy

    from .config import DjangoSettingsProvider
    from .utilities import DjangoPatchingUtilities
    from .runner import DjangoPatchingRunner

    django_settings_provider = DjangoSettingsProvider(settings=settings)

    generic_patch_software(
        settings=django_settings_provider,
        patching_registry=django_patching_registry,
        patching_utilities_class=DjangoPatchingUtilities,
        patching_runner_class=DjangoPatchingRunner,
        warnings_proxy=warnings_proxy,
    )



from __future__ import absolute_import, print_function, unicode_literals

import _test_utilities
import pytest


def test_fix_incoming_urls_submodule():
    from django.urls import (
        get_callable,
        RegexURLPattern,
        RegexURLResolver,
        NoReverseMatch,
    )

    assert get_callable


def test_fix_deletion_templatetags_future():
    from compat import render_to_string
    from django.templatetags.future import cycle, firstof

    rendered = render_to_string("core_tags/test_future_cycle_and_firstof.html")
    assert rendered.strip() == "row1\nA"


def test_fix_deletion_template_defaulttags_ssi():
    # already tested in "test_fix_deletion_templatetags_future_ssi()"
    from django.template.defaulttags import ssi

    assert callable(ssi)


def test_fix_behaviour_urls_resolvers_RegexURLPattern():

    from django.core.urlresolvers import RegexURLPattern

    has_lookup_str = hasattr(RegexURLPattern, "lookup_str")

    def dummy_view(request):
        return 72627

    pattern = RegexURLPattern("homepage/", dummy_view)
    assert pattern.callback is dummy_view
    pattern.add_prefix("muyprefix")
    assert pattern.callback is dummy_view
    if has_lookup_str:
        assert pattern.lookup_str.endswith(".dummy_view")  # complex on py3k

    pattern = RegexURLPattern("homepage/", "test_project.views.my_view")
    assert pattern.callback.__name__ == "my_view"
    if has_lookup_str:
        assert pattern.lookup_str == "test_project.views.my_view"
    pattern.add_prefix("myprefix")
    try:
        pattern.callback  # bad prefix now
    except ImportError:
        pass  # might or not raise, depending on django version (caching or not)
    if has_lookup_str:
        # our own "lookup_str" property bypasses the original, CACHED, one
        assert pattern.lookup_str == "myprefix.test_project.views.my_view"

    pattern = RegexURLPattern("homepage/", "my_view")
    with pytest.raises(ImportError):
        myvar = pattern.callback  # missing prefix
    if has_lookup_str:
        assert pattern.lookup_str == "my_view"  # missing prefix but works
    pattern.add_prefix("test_project.views")
    assert pattern.callback.__name__ == "my_view"
    if has_lookup_str:
        assert pattern.lookup_str == "test_project.views.my_view"


def test_fix_behaviour_core_urlresolvers_reverse_with_prefix():
    try:
        from django.core.urlresolvers import reverse
    except ImportError:
        from django.urls import reverse

    view = reverse("homepage")  # by view name
    assert view == "/homepage/"

    view = reverse("test_project.views.my_view")  # by dotted path
    assert view == "/my_view/"


def test_fix_behaviour_conf_urls_url():
    from django.conf.urls import url

    url(r"^admin2/", "test_project.views.my_view", name="test_admin_abc"),


def test_fix_deletion_conf_urls_patterns():
    import django.conf.urls
    from django.conf.urls import patterns, url

    patterns(
        "admin",
        (r"^admin1/", "test_project.views.my_view"),
        url(r"^admin2/", "test_project.views.my_view", name="test_admin_other"),
    )
    assert "patterns" in django.conf.urls.__all__


def test_fix_behaviour_template_smartif_OPERATORS_equals():
    from compat import render_to_string

    rendered = render_to_string("core_tags/test_smartif_operators.html", dict(a=3))
    assert rendered.strip() == "hello\nbye"



import os

import pytest
from compat_patcher_core.scaffolding import ensure_no_stdlib_warnings


def test_ensure_no_stdlib_warnings_in_package():
    import warnings  # This line will trigger checker error
    del warnings

    import django_compat_patcher

    pkg_root = os.path.dirname(django_compat_patcher.__file__)
    analysed_files = ensure_no_stdlib_warnings(pkg_root)
    assert len(analysed_files) >= 3, analysed_files

    test_root = os.path.dirname(__file__)
    with pytest.raises(ValueError, match="wrong phrase.*test_scaffolding.py"):
        ensure_no_stdlib_warnings(test_root)


def test_no_package_shadowing_in_tox():
    import django_compat_patcher

    package_dir = os.path.dirname(os.path.abspath(django_compat_patcher.__file__))
    if os.getenv("INSIDE_TOX_FOR_DCP") and ".tox" not in package_dir:
        raise RuntimeError("Wrong django_compat_patcher package used in Tox")



from __future__ import absolute_import, print_function, unicode_literals

import copy

from django.utils import six

from django_compat_patcher.deprecation import *


class MergeDict(object):
    """
    A simple class for creating new "virtual" dictionaries that actually look
    up values in more than one dictionary, passed in the constructor.

    If a key appears in more than one of the given dictionaries, only the
    first occurrence will be used.
    """

    def __init__(self, *dicts):
        warnings.warn('`MergeDict` is deprecated, use `dict.update()` '
                      'instead.', RemovedInDjango19Warning, 2)
        self.dicts = dicts

    def __bool__(self):
        return any(self.dicts)

    if six.PY2:
        def __nonzero__(self):
            return type(self).__bool__(self)

    def __getitem__(self, key):
        for dict_ in self.dicts:
            try:
                return dict_[key]
            except KeyError:
                pass
        raise KeyError(key)

    def __copy__(self):
        return self.__class__(*self.dicts)

    def get(self, key, default=None):
        try:
            return self[key]
        except KeyError:
            return default

    # This is used by MergeDicts of MultiValueDicts.
    def getlist(self, key):
        for dict_ in self.dicts:
            if key in dict_:
                return dict_.getlist(key)
        return []

    def _iteritems(self):
        seen = set()
        for dict_ in self.dicts:
            for item in six.iteritems(dict_):
                k = item[0]
                if k in seen:
                    continue
                seen.add(k)
                yield item

    def _iterkeys(self):
        for k, v in self._iteritems():
            yield k

    def _itervalues(self):
        for k, v in self._iteritems():
            yield v

    if six.PY3:
        items = _iteritems
        keys = _iterkeys
        values = _itervalues
    else:
        iteritems = _iteritems
        iterkeys = _iterkeys
        itervalues = _itervalues

        def items(self):
            return list(self.iteritems())

        def keys(self):
            return list(self.iterkeys())

        def values(self):
            return list(self.itervalues())

    def has_key(self, key):
        for dict_ in self.dicts:
            if key in dict_:
                return True
        return False

    __contains__ = has_key

    __iter__ = _iterkeys

    def copy(self):
        """Returns a copy of this object."""
        return self.__copy__()

    def __str__(self):
        '''
        Returns something like

            "{'key1': 'val1', 'key2': 'val2', 'key3': 'val3'}"

        instead of the generic "<object meta-data>" inherited from object.
        '''
        return str(dict(self.items()))

    def __repr__(self):
        '''
        Returns something like

            MergeDict({'key1': 'val1', 'key2': 'val2'}, {'key3': 'val3'})

        instead of generic "<object meta-data>" inherited from object.
        '''
        dictreprs = ', '.join(repr(d) for d in self.dicts)
        return '%s(%s)' % (self.__class__.__name__, dictreprs)


class SortedDict(dict):
    """
    A dictionary that keeps its keys in the order in which they're inserted.
    """

    def __new__(cls, *args, **kwargs):
        instance = super(SortedDict, cls).__new__(cls, *args, **kwargs)
        instance.keyOrder = []
        return instance

    def __init__(self, data=None):
        warnings.warn(
            "SortedDict is deprecated and will be removed in Django 1.9.",
            RemovedInDjango19Warning, stacklevel=2
        )
        if data is None or isinstance(data, dict):
            data = data or []
            super(SortedDict, self).__init__(data)
            self.keyOrder = list(data) if data else []
        else:
            super(SortedDict, self).__init__()
            super_set = super(SortedDict, self).__setitem__
            for key, value in data:
                # Take the ordering from first key
                if key not in self:
                    self.keyOrder.append(key)
                # But override with last value in data (dict() does this)
                super_set(key, value)

    def __deepcopy__(self, memo):
        return self.__class__([(key, copy.deepcopy(value, memo))
                               for key, value in self.items()])

    def __copy__(self):
        # The Python's default copy implementation will alter the state
        # of self. The reason for this seems complex but is likely related to
        # subclassing dict.
        return self.copy()

    def __setitem__(self, key, value):
        if key not in self:
            self.keyOrder.append(key)
        super(SortedDict, self).__setitem__(key, value)

    def __delitem__(self, key):
        super(SortedDict, self).__delitem__(key)
        self.keyOrder.remove(key)

    def __iter__(self):
        return iter(self.keyOrder)

    def __reversed__(self):
        return reversed(self.keyOrder)

    def pop(self, k, *args):
        result = super(SortedDict, self).pop(k, *args)
        try:
            self.keyOrder.remove(k)
        except ValueError:
            # Key wasn't in the dictionary in the first place. No problem.
            pass
        return result

    def popitem(self):
        result = super(SortedDict, self).popitem()
        self.keyOrder.remove(result[0])
        return result

    def _iteritems(self):
        for key in self.keyOrder:
            yield key, self[key]

    def _iterkeys(self):
        for key in self.keyOrder:
            yield key

    def _itervalues(self):
        for key in self.keyOrder:
            yield self[key]

    if six.PY3:
        items = _iteritems
        keys = _iterkeys
        values = _itervalues
    else:
        iteritems = _iteritems
        iterkeys = _iterkeys
        itervalues = _itervalues

        def items(self):
            return [(k, self[k]) for k in self.keyOrder]

        def keys(self):
            return self.keyOrder[:]

        def values(self):
            return [self[k] for k in self.keyOrder]

    def update(self, dict_):
        for k, v in six.iteritems(dict_):
            self[k] = v

    def setdefault(self, key, default):
        if key not in self:
            self.keyOrder.append(key)
        return super(SortedDict, self).setdefault(key, default)

    def copy(self):
        """Returns a copy of this object."""
        # This way of initializing the copy means it works for subclasses, too.
        return self.__class__(self)

    def __repr__(self):
        """
        Replaces the normal dict.__repr__ with a version that returns the keys
        in their sorted order.
        """
        return '{%s}' % ', '.join('%r: %r' % (k, v) for k, v in six.iteritems(self))

    def clear(self):
        super(SortedDict, self).clear()
        self.keyOrder = []



from __future__ import absolute_import, print_function, unicode_literals

import _test_utilities

import os

import pytest


class MockRequest:
    pass


def test_fix_deletion_templatetags_future_url():
    from compat import render_to_string

    rendered = render_to_string("core_tags/test_future_url.html")
    assert rendered.strip() == "/homepage/"

    rendered = render_to_string("core_tags/test_defaulttags_url.html")
    assert rendered.strip() == "/homepage/"


def test_fix_deletion_templatetags_future_ssi():
    from compat import render_to_string

    filepath = os.path.abspath(__file__)

    rendered = render_to_string(
        "core_tags/test_future_ssi.html", dict(filepath=filepath)
    )
    assert "test_keep_templatetags_future_ssi()" in rendered

    rendered = render_to_string(
        "core_tags/test_defaulttags_ssi.html", dict(filepath=filepath)
    )
    assert "test_keep_templatetags_future_ssi()" in rendered


def test_fix_deletion_forms_fields_IPAddressField():
    from django.forms import ValidationError
    from django.forms.fields import IPAddressField

    def assertEqual(a, b):
        assert a == b

    def assertFormErrors(expected, the_callable, *args, **kwargs):
        try:
            the_callable(*args, **kwargs)
            raise ValueError(
                "Testing the 'clean' method on %s failed to raise a ValidationError."
            )
        except ValidationError as e:
            assert e.messages == expected

    f = IPAddressField()
    assertFormErrors(["This field is required."], f.clean, "")
    assertFormErrors(["This field is required."], f.clean, None)
    assertEqual(f.clean(" 127.0.0.1"), "127.0.0.1")
    assertFormErrors(["Enter a valid IPv4 address."], f.clean, "foo")
    assertFormErrors(["Enter a valid IPv4 address."], f.clean, "127.0.0.")
    assertFormErrors(["Enter a valid IPv4 address."], f.clean, "1.2.3.4.5")
    assertFormErrors(["Enter a valid IPv4 address."], f.clean, "256.125.1.5")

    f = IPAddressField(required=False)
    assertEqual(f.clean(""), "")
    assertEqual(f.clean(None), "")
    assertEqual(f.clean(" 127.0.0.1"), "127.0.0.1")
    assertFormErrors(["Enter a valid IPv4 address."], f.clean, "foo")
    assertFormErrors(["Enter a valid IPv4 address."], f.clean, "127.0.0.")
    assertFormErrors(["Enter a valid IPv4 address."], f.clean, "1.2.3.4.5")
    assertFormErrors(["Enter a valid IPv4 address."], f.clean, "256.125.1.5")

    from django.db.models.fields import IPAddressField as ModelIPAddressField

    res = ModelIPAddressField().formfield()
    assert isinstance(res, IPAddressField)


def test_fix_deletion_core_handlers_wsgi_WSGIRequest_REQUEST():
    from django.test.client import RequestFactory

    factory = RequestFactory()

    request = factory.get("/homepage/?abcd=66")
    assert "abc" not in request.REQUEST

    request = factory.get("/homepage/?abc")
    assert request.REQUEST["abc"] == ""

    request = factory.get("/homepage/?abc=6%26")
    assert request.REQUEST["abc"] == "6&"

    request = factory.post("/homepage/?abc=66", data=dict(abc="aju"))
    assert request.REQUEST["abc"] == "aju"  # POST takes precedence over GET


def test_fix_deletion_contrib_admin_ModelAdmin_get_formsets():
    from django.contrib.admin import ModelAdmin
    from test_project.models import SimpleModel
    from django.contrib.admin import AdminSite

    assert hasattr(ModelAdmin, "get_formsets")

    ma = ModelAdmin(SimpleModel, AdminSite())

    assert ma.get_fieldsets(request=MockRequest()) == [
        (None, {"fields": ["name", "age", "is_active"]})
    ]


def test_fix_deletion_utils_datastructures_MergeDict():
    from django.utils.datastructures import MergeDict

    MergeDict()


def test_fix_deletion_utils_datastructures_SortedDict():
    from django.utils.datastructures import SortedDict

    SortedDict()


def test_fix_deletion_utils_importlib():
    import django.utils.importlib
    from django.utils import importlib

    csv_module = importlib.import_module("csv")
    import csv

    assert csv_module is csv


def test_fix_deletion_utils_tzinfo():
    import django.utils.tzinfo
    from django.utils import tzinfo

    assert tzinfo.FixedOffset(35)


def test_fix_deletion_utils_dictconfig():
    import django.utils.dictconfig
    from django.utils import dictconfig

    assert dictconfig.valid_ident("myident")


def test_fix_deletion_utils_functional_memoize():
    import django.utils.functional
    from django.utils.functional import memoize

    cache = {}

    def myfun(myarg):
        return myarg

    myfun = memoize(myfun, cache, num_args=1)
    myfun(3)
    myfun(4)
    assert cache == {(3,): 3, (4,): 4}


def test_fix_deletion_utils_unittest():
    import django.utils.unittest
    from django.utils import unittest

    assert callable(unittest.TestCase)


def test_fix_deletion_core_management_base_AppCommand_handle_app():
    from django.core.management.base import AppCommand

    class DummyAppconfig(object):
        label = "my dummy app"
        models_module = "fake_models_module"

    appconfig = DummyAppconfig()

    class DummyAppCommand(AppCommand):
        def handle_app(self, models_module, **options):
            return 33

    res = DummyAppCommand().handle_app_config(appconfig, other_arg=12)
    assert res == 33


def test_fix_deletion_contrib_sites_models_RequestSite():
    from django.contrib.sites.models import RequestSite
    from django.contrib.sites.requests import RequestSite as RealRequestSite

    assert callable(RequestSite)
    assert issubclass(RequestSite, RealRequestSite)


def test_fix_deletion_contrib_sites_models_get_current_site():
    from django.core.exceptions import ImproperlyConfigured

    class request:
        SITE_ID = 1

    from django.contrib.sites.models import get_current_site

    with pytest.raises(ImproperlyConfigured):  # DCP tests have no DB access for now
        assert get_current_site(request)



import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "==dfic-%x71zf%q9468&z#h#gsao=_oxg)jg+^+zn4)7a*ihe5"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ["*"]

INSTALLED_APPS = []  # EMPTY



#!/usr/bin/env python
import os
import sys

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "test_project.settings")

import _test_utilities  # bootstraps django-compat-patcher

if __name__ == "__main__":

    from django.core.management import execute_from_command_line

    execute_from_command_line(sys.argv)



from __future__ import absolute_import, print_function, unicode_literals

from functools import partial

from ..deprecation import *
from ..registry import register_django_compatibility_fixer

# for backward-compatibility fixers
django1_8_bc_fixer = partial(
    register_django_compatibility_fixer,
    fixer_reference_version="1.8",
    fixer_applied_from_version="1.8",
)


@django1_8_bc_fixer()
def fix_outsourcing_contrib_comments(utils):
    """
    Keep 'django.contrib.comments' as an import alias for the now external package
    'django_comments' (django-contrib-comments on pypi) ; the latter must be installed separately.
    """
    utils.inject_import_alias(
        alias_name="django.contrib.comments", real_name="django_comments"
    )



"""
WSGI config for test_project project.
"""

import os

from django.core.wsgi import get_wsgi_application

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "test_project.settings")

application = get_wsgi_application()



from __future__ import absolute_import, print_function, unicode_literals

from compat_patcher_core.utilities import PatchingUtilities
from compat_patcher_core import SkipFixerException


class DjangoPatchingUtilities(PatchingUtilities):
    @staticmethod
    def skip_if_app_not_installed(app_name):
        """
        Raises a SkipFixerException if app_name is not enabled in Django settings.
        """
        from django.conf import settings

        if app_name not in settings.INSTALLED_APPS:
            raise SkipFixerException("%s is not enabled in INSTALLED_APPS" % app_name)
